TShadow = 20;

lambda = 5.646965634867457887779456099391322297847861701252815127709856919302064392661481081041338926748410690;
eta = 0.06114349344118513700;
zeta = .45463331959776981000;

ShockVariables = { 'Z', 'Phi', 'Theta', 'GN', 'LI' };

Options = optimset( 'MaxIter', 4096, 'LargeScale', 'off' );
ZeroVec = zeros( TShadow, 1 );

gIShadow = zeros( 200, TShadow );

for VariableIndex = 1:5
	for Sign = -1:2:1
		if Sign > 0
			SGNT = '_P';
		else
			SGNT = '_M';
		end
		figure( 'PaperUnits', 'centimeters', 'Renderer', 'painters', 'Units', 'centimeters' );

		SN = ShockVariables{ VariableIndex };

		alpha = ZeroVec;

		for Magnitude = 0:1:1
			MG = Magnitude * Sign;
			gI = GetIRF( 'LAG_0_gI', SN, MG );

			for indx = 0:( TShadow - 1 )
				gIShadow( :, indx + 1 ) = GetRelIRF( 'LAG_0_gI', [ 'shadowI_' sprintf( '%d', indx ) ], 1 );
			end

			% alpha' * ( gI( 1:TShadow ) + gIShadow( 1:TShadow, : ) * alpha ) = 0
			% => 1/2 * alpha' * ( gIShadow( 1:TShadow, : ) + gIShadow( 1:TShadow, : )' ) * alpha + gI' * alpha = 0
			% gI + gIShadow * alpha >= 0
			% => -gIShadow * alpha <= gI

			[ alpha, FoundValue ] =  quadprog( gIShadow( 1:TShadow, : ) + gIShadow( 1:TShadow, : )', gI( 1:TShadow ), -gIShadow, gI, [], [], ZeroVec, [], alpha, Options );

			% gI = gI + gIShadow * alpha;

			if FoundValue < 1e-16
				fprintf( 1, [ 'Solved problem with ' SN ' shock at MG %f.\nShock binds in %d periods.\n\n' ], MG, sum( alpha > 1e-16 ) );

				gNR = GetRelIRF( 'log_GN', SN, MG );
				nR = cumsum( gNR );
				aHatR = GetRelBoundedIRF( 'a', SN, MG, alpha );
				aNHatR = GetRelBoundedIRF( 'aN', SN, MG, alpha );
				gAsR = GetRelBoundedIRF( 'gAs', SN, MG, alpha );
				asR = cumsum( gAsR );
				LAG_aHatR = [ 0; aHatR( 1:199 ) ];
				LAG_aNHatR = [ 0; aNHatR( 1:199 ) ];
				gAR = aHatR - LAG_aHatR + gAsR;
				gANR = aNHatR - LAG_aNHatR + gAsR;
				aR = cumsum( gAR );
				aNR = cumsum( gANR );
				muP = exp( GetBoundedIRF( 'log_1PmuP', SN, MG, alpha ) ) - 1;
				mu =  exp( GetBoundedIRF( 'log_1Pmu', SN, MG, alpha ) ) - 1;
				muPMean = exp( GetMean( 'log_1PmuP' ) ) - 1;
				muMean = exp( GetMean( 'log_1Pmu' ) ) - 1;
				LAG_mu = [ muMean; mu( 1:199 ) ];
				log_muPR = log( muP ) - log( muPMean );
				log_muR = log( mu ) - log( muMean );
				wR = aR - log( 1 + LAG_mu ) + log( 1 + muMean );
				jPR = GetRelBoundedIRF( 'jP', SN, MG, alpha );
				jP = jPR + GetMean( 'jP' );
				iHatR = GetRelBoundedIRF( 'i', SN, MG, alpha );
				LAG_iHatR = [ 0; iHatR( 1:199 ) ];
				gIR = gNR - zeta * gAsR + iHatR - LAG_iHatR;
				gIMean = GetMean( 'log_GN' ) - zeta * GetMean( 'gAs' );
				gI = gIR + gIMean;
				iR = cumsum( gIR );
				Fw = exp( GetBoundedIRF( 'log_Fw', SN, MG, alpha ) );
				FwMean = exp( GetMean( 'log_Fw' ) );
				log_FwR = log( Fw ) - log( FwMean );
				FLR = GetBoundedIRF( 'FLR', SN, MG, alpha );
				FlRR = log( FLR ) - log( GetMean( 'FLR' ) );
				lSR = GetRelBoundedIRF( 'lS', SN, MG, alpha ) + nR;
				yR = GetRelBoundedIRF( 'y', SN, MG, alpha ) + aR + nR;
				ResExpR = iHatR + nR + wR + jPR + FlRR + log( 1 - Fw ) - log( 1 - FwMean );
				ResExpI = GetRelIRF( 'log_LI', SN, MG ) + iHatR + nR + wR + log( max( 1e-32, 1 - 1 ./ exp( gI ) ) ) - log( 1 - 1 ./ exp( gIMean ) );
				rR = GetRelBoundedIRF( 'r', SN, MG, alpha );

				if min( jP ) < 1
					error( 'JMP:TooFewFirms', [ 'The number of firms went below 1 following the ' SN ' shock at MG %f.\n\n' ], MG );
				end
				if min( FLR ) < 0
					error( 'JMP:NegativeResearch', [ 'Research went negative following the ' SN ' shock at MG %f.\n\n' ], MG );
				end

				Tirfs = cell( 3, 5 );

				Tirfs{ 1, 1 } = asR;
				Tirfs{ 2, 1 } = aNR;
				Tirfs{ 3, 1 } = aR;
				Tirfs{ 1, 2 } = log_muPR;
				Tirfs{ 2, 2 } = log_muR;
				Tirfs{ 3, 2 } = wR;
				Tirfs{ 1, 3 } = jPR;
				Tirfs{ 2, 3 } = iR;
				Tirfs{ 3, 3 } = log_FwR;
				Tirfs{ 1, 4 } = FlRR;
				Tirfs{ 2, 4 } = lSR;
				Tirfs{ 3, 4 } = yR;
				Tirfs{ 1, 5 } = ResExpR;
				Tirfs{ 2, 5 } = ResExpI;
				Tirfs{ 3, 5 } = rR;

				for i = 1:3
					for j = 1:5
						LastImage = subplot( 3, 5, sub2ind( [5 3], j, i ) );
					hold( LastImage );
						if abs( MG ) == 1.0
							LString = 'k-';
						else
							LString = 'k:';
						end
						if ~isreal( Tirfs{ i, j } )
							error( 'JMP:Complex', [ 'Complex value encountered in response to ' SN ' shock at MG %f.\n\n' ], MG );
						end
						plot( 0:200, [0; 100 * exp( Tirfs{ i, j } ) - 100], LString, 'LineWidth', 1 );
						axis square;
						set( gca, 'XLim', [0, 200] );
						set( gca, 'XTick', [0, 40, 80, 120, 160, 200] );
						set( gca, 'FontSize', 24 );
						set( gca, 'LineWidth', 1 );
						hold( LastImage );
					end
				end        
			else
				error( 'JMP:NoSolution', [ 'Failed to solve problem with ' SN ' shock at MG %f.\nFound value: %f\n\n' ], MG, FoundValue );
			end
		end

		set( gcf, 'PaperPositionMode', 'manual' );
		Width = 100;% 19 * 2;
		set( gcf, 'PaperSize', [ Width, Width * 3 / 5 ] );
		set( gcf, 'PaperPosition', [ 0, 0, Width, Width * 3 / 5 ] );
		print( '-depsc2', [SN SGNT], '-painters', '-r1116' );

		set( gcf, 'Position', [ 0, 0, Width, Width * 3 / 5 ] );
		ActualPosition = get( gcf, 'Position' );
		while ActualPosition( 4 ) ~= ActualPosition( 3 ) * 3 / 5;
			set( gcf, 'Position', [ ActualPosition( 1:2 ), min( ActualPosition( 3 ), ActualPosition( 4 ) * 5 / 3 ), min( ActualPosition( 3 ) * 3 / 5, ActualPosition( 4 ) ) ] );
			ActualPosition = get( gcf, 'Position' );
		end
		set( gcf, 'Resize', 'off' );
		set( LastImage, 'Units', 'inches' );
		ImagePosition = get( LastImage, 'Position' );
		Resolution = ceil( 500 / ImagePosition( 3 ) );
		print( '-dmeta', [SN SGNT], '-painters', ['-r' sprintf( '%d', Resolution ) ] );
		close;
	end
end